let tabIds = new Set();

function tabIdsFromQuerying(callback) {
    chrome.tabs.query({}, function(tabs) {
        if ( !tabs ) { return; }
        let tabIds = [];
        for ( var tab of tabs ) {
            tabIds.push(tab.id);
        }
        callback(tabIds);
    });
}

tabIdsFromQuerying(function(ids) {
    for ( let id of ids ) {
        tabIds.add(id);
    }
});

function logTabIdsFromQuerying() {
    tabIdsFromQuerying(function(ids) {
        console.log('all tab ids from tabs.query():', ids);
    });
}

// tabs

chrome.tabs.onUpdated.addListener(function(tabId, info) {
    tabIds.add(tabId);
    console.log('tabs.onUpdated:' , tabId, JSON.stringify(info));
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});

chrome.tabs.onRemoved.addListener(function(tabId, info) {
    tabIds.delete(tabId);
    console.log('tabs.onRemoved:', tabId, JSON.stringify(info));
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});

// webNavigation

chrome.webNavigation.onCreatedNavigationTarget.addListener(function(details) {
    tabIds.add(details.tabId);
    console.log('webNavigation.onCreatedNavigationTarget:' , details.tabId);
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});

chrome.webNavigation.onCommitted.addListener(function(details) {
    tabIds.add(details.tabId);
    console.log('webNavigation.onCommitted:' , details.tabId);
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});

// webRequest

chrome.webRequest.onBeforeRequest.addListener(function(details) {
    tabIds.add(details.tabId);
    console.log('webRequest.onBeforeRequest:' , details.tabId);
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
}, { urls: ['<all_urls>' ], types: [ 'main_frame' ]});



// added by mise

chrome.tabs.onAttached.addListener(function(tabId, info) {
    tabIds.add(tabId);
    console.log('tabs.onAttached:' , tabId, JSON.stringify(info));
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});


chrome.tabs.onDetached.addListener(function(tabId, info) {
    tabIds.delete(tabId);
    console.log('tabs.onDetached:' , tabId, JSON.stringify(info));
//    console.log('tab ids from event listening:', Array.from(tabIds));
    logTabIdsFromQuerying();
});
